@@ -30,7 +30,7 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID (PKG_VER "$Id: login.c,v 1.25 2002/01/05 15:41:43 kloczek Exp $")
+RCSID (PKG_VER "$Id: login.c,v 1.37 2004/08/04 10:33:43 kloczek Exp $")
 #include "prototypes.h"
 #include "defines.h"
 #include <sys/stat.h>
@@ -38,11 +38,6 @@ RCSID (PKG_VER "$Id: login.c,v 1.25 2002/01/05 15:41:43 kloczek Exp $")
 #include <errno.h>
 #include <pwd.h>
 #include <grp.h>
-#if HAVE_UTMPX_H
-#include <utmpx.h>
-#else
-#include <utmp.h>
-#endif
 #include <signal.h>
 #if HAVE_LASTLOG_H
 #include <lastlog.h>
@@ -53,11 +48,6 @@ RCSID (PKG_VER "$Id: login.c,v 1.25 2002/01/05 15:41:43 kloczek Exp $")
 #include "failure.h"
 #include "pwauth.h"
 #include "getdef.h"
-#include "dialchk.h"
-#ifdef SVR4_SI86_EUA
-#include <sys/proc.h>
-#include <sys/sysi86.h>
-#endif
 #ifdef RADIUS
 /*
  * Support for RADIUS authentication based on a hacked util-linux login
@@ -99,14 +89,16 @@ static pam_handle_t *pamh = NULL;
 
 const char *hostname = "";
 
-struct passwd pwent;
+static struct passwd pwent;
 
 #if HAVE_UTMPX_H
-struct utmpx utxent, failent;
-struct utmp utent;
+extern	struct	utmpx	utxent;
+struct utmpx failent;
 #else
-struct utmp utent, failent;
+struct utmp failent;
 #endif
+extern	struct	utmp	utent;
+
 struct lastlog lastlog;
 static int pflg = 0;
 static int fflg = 0;
@@ -180,7 +172,7 @@ static RETSIGTYPE alarm_handler (int);
 
 static void usage (void)
 {
-	fprintf (stderr, _("usage: %s [-p] [name]\n"), Prog);
+	fprintf (stderr, _("Usage: %s [-p] [name]\n"), Prog);
 	if (!amroot)
 		exit (1);
 	fprintf (stderr, _("       %s [-p] [-h host] [-f name]\n"), Prog);
@@ -230,15 +222,10 @@ static void setup_tty (void)
 	termio.c_iflag &= ~IXANY;
 	termio.c_oflag |= (XTABS | OPOST | ONLCR);
 #endif
-#if 0
-	termio.c_cc[VERASE] = getdef_num ("ERASECHAR", '\b');
-	termio.c_cc[VKILL] = getdef_num ("KILLCHAR", '\025');
-#else
 	/* leave these values unchanged if not specified in login.defs */
 	termio.c_cc[VERASE] =
 	    getdef_num ("ERASECHAR", termio.c_cc[VERASE]);
 	termio.c_cc[VKILL] = getdef_num ("KILLCHAR", termio.c_cc[VKILL]);
-#endif
 
 	/*
 	 * ttymon invocation prefers this, but these settings won't come into
@@ -579,10 +566,10 @@ int main (int argc, char **argv)
 		else if (amroot)
 			memzero (utent.ut_host, sizeof utent.ut_host);
 #endif
-		if (hflg && fflg) {
-			reason = PW_RLOGIN;
+		if (fflg)
 			preauth_flag++;
-		}
+		if (hflg)
+			reason = PW_RLOGIN;
 #ifdef RLOGIN
 		if (rflg
 		    && do_rlogin (hostname, username, sizeof username,
@@ -634,36 +621,10 @@ int main (int argc, char **argv)
 			if (rflg || fflg)
 				usage ();
 
-#ifdef SVR4
-			/*
-			 * The "-h" option can't be used with a command-line username,
-			 * because telnetd invokes us as: login -h host TERM=...
-			 */
-
-			if (!hflg)
-#endif
-			{
-				STRFCPY (username, argv[optind]);
-				strzero (argv[optind]);
-				++optind;
-			}
+			STRFCPY (username, argv[optind]);
+			strzero (argv[optind]);
+			++optind;
 		}
-#ifdef SVR4
-		/*
-		 * check whether ttymon has done the prompt for us already
-		 */
-
-		{
-			char *ttymon_prompt;
-
-			if ((ttymon_prompt = getenv ("TTYPROMPT")) != NULL
-			    && (*ttymon_prompt != 0)) {
-				/* read name, without prompt */
-				login_prompt ((char *) 0, username,
-					      sizeof username);
-			}
-		}
-#endif				/* SVR4 */
 		if (optind < argc)	/* now set command line variables */
 			set_env (argc - optind, &argv[optind]);
 
@@ -824,6 +785,11 @@ int main (int argc, char **argv)
 		setpwent ();
 		pwd = getpwnam (pam_user);
 
+		if (fflg) {
+			retcode = pam_acct_mgmt(pamh, 0);
+			PAM_FAIL_CHECK;
+		}
+
 		if (!pwd || setup_groups (pwd))
 			exit (1);
 
@@ -909,19 +875,6 @@ int main (int argc, char **argv)
 		if (preauth_flag)
 			goto auth_ok;
 
-		/*
-		 * No password prompt if logging in from listed ttys (local
-		 * console). Passwords don't help much if you have physical
-		 * access to the hardware anyway...  Suggested by Pavel
-		 * Machek <pavel@bug.ucw.cz>.
-		 * NOTE: password still required for root logins!
-		 */
-		if (pwd && (pwent.pw_uid != 0)
-		    && is_listed ("NO_PASSWORD_CONSOLE", tty, 0)) {
-			temp_pw[0] = '\0';
-			pwent.pw_passwd = temp_pw;
-		}
-
 		if (pw_auth
 		    (pwent.pw_passwd, username, reason,
 		     (char *) 0) == 0)
@@ -972,18 +925,6 @@ int main (int argc, char **argv)
 		}
 #endif
 
-		if (getdef_bool ("DIALUPS_CHECK_ENAB")) {
-			alarm (30);
-
-			if (!dialcheck (tty, pwent.pw_shell[0] ?
-					pwent.pw_shell : "/bin/sh")) {
-				SYSLOG ((LOG_WARN,
-					 "invalid dialup password for `%s' on `%s'",
-					 username, tty));
-				failed = 1;
-			}
-		}
-
 		if (!failed && pwent.pw_name && pwent.pw_uid == 0
 		    && !is_console) {
 			SYSLOG ((LOG_CRIT, "ILLEGAL ROOT LOGIN %s",
@@ -1018,10 +959,18 @@ int main (int argc, char **argv)
 
 #if HAVE_UTMPX_H
 			failent = utxent;
-			gettimeofday (&(failent.ut_tv), NULL);
+			if (sizeof (failent.ut_tv) == sizeof (struct timeval))
+			  gettimeofday ((struct timeval *) &failent.ut_tv, NULL);
+			else
+			  {
+				struct timeval tv;
+				gettimeofday (&tv, NULL);
+				failent.ut_tv.tv_sec = tv.tv_sec;
+				failent.ut_tv.tv_usec = tv.tv_usec;
+			  }
 #else
 			failent = utent;
-			time (&failent.ut_time);
+			failent.ut_time = time (NULL);
 #endif
 			if (pwd) {
 				failent_user = pwent.pw_name;
@@ -1122,10 +1071,6 @@ int main (int argc, char **argv)
 			   hostname);
 #endif
 
-#ifdef SVR4_SI86_EUA
-	sysi86 (SI86LIMUSER, EUA_ADD_USER);	/* how do we test for fail? */
-#endif
-
 #ifndef USE_PAM			/* PAM handles this as well */
 	/*
 	 * Have to do this while we still have root privileges, otherwise we
@@ -1142,16 +1087,6 @@ int main (int argc, char **argv)
 				pwent = *pwd;
 		}
 	}
-#else
-#ifdef	ATT_AGE
-	if (pwent.pw_age && pwent.pw_age[0]) {
-		if (expire (&pwent)) {
-			pwd = getpwnam (username);
-			if (pwd)
-				pwent = *pwd;
-		}
-	}
-#endif				/* ATT_AGE */
 #endif				/* SHADOWPWD */
 
 #ifdef RADIUS
@@ -1269,18 +1204,19 @@ int main (int argc, char **argv)
 		}
 		if (getdef_bool ("LASTLOG_ENAB")
 		    && lastlog.ll_time != 0) {
+			time_t ll_time = lastlog.ll_time;
 #ifdef HAVE_STRFTIME
 			strftime (ptime, sizeof (ptime),
 				  "%a %b %e %H:%M:%S %z %Y",
-				  localtime (&lastlog.ll_time));
+				  localtime (&ll_time));
 			printf (_("Last login: %s on %s"),
 				ptime, lastlog.ll_line);
 #else
 			printf (_("Last login: %.19s on %s"),
-				ctime (&lastlog.ll_time),
+				ctime (&ll_time),
 				lastlog.ll_line);
 #endif
-#ifdef HAVE_LL_HOST		/* SVR4 || __linux__ || SUN4 */
+#ifdef HAVE_LL_HOST		/* __linux__ || SUN4 */
 			if (lastlog.ll_host[0])
 				printf (_(" from %.*s"),
 					(int) sizeof lastlog.
@@ -1368,5 +1304,6 @@ int main (int argc, char **argv)
 		shell (tmp, pwent.pw_shell);	/* fake shell */
 	}
 	shell (pwent.pw_shell, (char *) 0);	/* exec the shell finally. */
-	 /*NOTREACHED*/ return 0;
+	/* NOT REACHED */
+	return 0;
 }
