@@ -30,7 +30,7 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID (PKG_VER "$Id: su.c,v 1.22 2002/03/08 04:30:28 kloczek Exp $")
+RCSID (PKG_VER "$Id: su.c,v 1.27 2004/10/23 23:19:07 kloczek Exp $")
 #include <sys/types.h>
 #include <stdio.h>
 #ifdef USE_PAM
@@ -134,6 +134,112 @@ static void su_failure (const char *tty)
 	exit (1);
 }
 
+#ifdef USE_PAM
+static int caught = 0;
+
+/* Signal handler for parent process later */
+static void su_catch_sig (int sig)
+{
+	++caught;
+}
+
+/* This I ripped out of su.c from sh-utils after the Mandrake pam patch
+ * have been applied.  Some work was needed to get it integrated into
+ * su.c from shadow.
+ */
+static void run_shell (const char *shellstr, char *args[], int doshell)
+{
+	int child;
+	sigset_t ourset;
+	int status;
+	int ret;
+
+	child = fork ();
+	if (child == 0) {	/* child shell */
+		pam_end (pamh, PAM_SUCCESS);
+
+		if (doshell)
+			shell (shellstr, (char *) args[0]);
+		else
+			(void) execv (shellstr, (char **) args);
+		{
+			int exit_status = (errno == ENOENT ? 127 : 126);
+
+			exit (exit_status);
+		}
+	} else if (child == -1) {
+		(void) fprintf (stderr, "%s: Cannot fork user shell\n",
+				Prog);
+		SYSLOG ((LOG_WARN, "Cannot execute %s", pwent.pw_shell));
+		closelog ();
+		exit (1);
+	}
+	/* parent only */
+	sigfillset (&ourset);
+	if (sigprocmask (SIG_BLOCK, &ourset, NULL)) {
+		(void) fprintf (stderr, "%s: signal malfunction\n", Prog);
+		caught = 1;
+	}
+	if (!caught) {
+		struct sigaction action;
+
+		action.sa_handler = su_catch_sig;
+		sigemptyset (&action.sa_mask);
+		action.sa_flags = 0;
+		sigemptyset (&ourset);
+
+		if (sigaddset (&ourset, SIGTERM)
+		    || sigaddset (&ourset, SIGALRM)
+		    || sigaction (SIGTERM, &action, NULL)
+		    || sigprocmask (SIG_UNBLOCK, &ourset, NULL)
+		    ) {
+			fprintf (stderr,
+				 "%s: signal masking malfunction\n", Prog);
+			caught = 1;
+		}
+	}
+
+	if (!caught) {
+		do {
+			int pid;
+
+			pid = waitpid (-1, &status, WUNTRACED);
+
+			if (WIFSTOPPED (status)) {
+				kill (getpid (), SIGSTOP);
+				/* once we get here, we must have resumed */
+				kill (pid, SIGCONT);
+			}
+		} while (WIFSTOPPED (status));
+	}
+
+	if (caught) {
+		fprintf (stderr, "\nSession terminated, killing shell...");
+		kill (child, SIGTERM);
+	}
+
+	ret = pam_close_session (pamh, 0);
+	if (ret != PAM_SUCCESS) {
+		SYSLOG ((LOG_ERR, "pam_close_session: %s",
+			 pam_strerror (pamh, ret)));
+		fprintf (stderr, "%s: %s\n", Prog,
+			 pam_strerror (pamh, ret));
+		pam_end (pamh, ret);
+		exit (1);
+	}
+
+	ret = pam_end (pamh, PAM_SUCCESS);
+
+	if (caught) {
+		sleep (2);
+		kill (child, SIGKILL);
+		fprintf (stderr, " ...killed.\n");
+		exit (-1);
+	}
+
+	exit (WEXITSTATUS (status));
+}
+#endif
 
 /*
  * su - switch user id
@@ -152,6 +258,7 @@ static void su_failure (const char *tty)
 int main (int argc, char **argv)
 {
 	char *cp;
+	char **envcp;
 	const char *tty = 0;	/* Name of tty SU is run from        */
 	int doshell = 0;
 	int fakelogin = 0;
@@ -252,6 +359,16 @@ int main (int argc, char **argv)
 		 */
 		if ((cp = getenv ("TERM")))
 			addenv ("TERM", cp);
+#ifndef USE_PAM
+		/*
+		 * Also leave DISPLAY and XAUTHORITY if present, else
+		 * pam_xauth will not work.
+		 */
+		if ((cp = getenv ("DISPLAY")))
+			addenv ("DISPLAY", cp);
+		if ((cp = getenv ("XAUTHORITY")))
+			addenv ("XAUTHORITY", cp);
+#endif				/* !USE_PAM */
 	} else {
 		while (*envp)
 			addenv (*envp++, NULL);
@@ -464,14 +581,6 @@ int main (int argc, char **argv)
 				 "Expired account %s", name));
 			su_failure (tty);
 		}
-#else
-#if defined(ATT_AGE)
-		if (pwent.pw_age[0] && isexpired (&pwent)) {
-			SYSLOG ((pwent.pw_uid ? LOG_WARN : LOG_CRIT,
-				 "Expired account %s", name));
-			su_failure (tty);
-		}
-#endif				/* ATT_AGE */
 #endif
 	}
 
@@ -494,9 +603,7 @@ int main (int argc, char **argv)
 
 	signal (SIGINT, SIG_DFL);
 	cp = getdef_str ((pwent.pw_uid == 0) ? "ENV_SUPATH" : "ENV_PATH");
-#if 0
-	addenv (cp ? cp : "PATH=/bin:/usr/bin", NULL);
-#else
+
 	/* XXX very similar code duplicated in libmisc/setupenv.c */
 	if (!cp) {
 		addenv ("PATH=/bin:/usr/bin", NULL);
@@ -505,9 +612,11 @@ int main (int argc, char **argv)
 	} else {
 		addenv ("PATH", cp);
 	}
-#endif
 
+/* setup the environment for pam later on, else we run into auth problems */
+#ifndef USE_PAM
 	environ = newenvp;	/* make new environment active */
+#endif
 
 	if (getenv ("IFS"))	/* don't export user IFS ... */
 		addenv ("IFS= \t\n", NULL);	/* ... instead, set a safe IFS */
@@ -555,16 +664,37 @@ int main (int argc, char **argv)
 		exit (1);
 	}
 
+	ret = pam_open_session (pamh, 0);
+	if (ret != PAM_SUCCESS) {
+		SYSLOG ((LOG_ERR, "pam_open_session: %s",
+			 pam_strerror (pamh, ret)));
+		fprintf (stderr, "%s: %s\n", Prog,
+			 pam_strerror (pamh, ret));
+		pam_end (pamh, ret);
+		exit (1);
+	}
+
+	/* we need to setup the environment *after* pam_open_session(),
+	 * else the UID is changed before stuff like pam_xauth could
+	 * run, and we cannot access /etc/shadow and co
+	 */
+	environ = newenvp;	/* make new environment active */
+
+	/* update environment with all pam set variables */
+	envcp = pam_getenvlist (pamh);
+	if (envcp) {
+		while (*envcp) {
+			putenv (*envcp);
+			envcp++;
+		}
+	}
+
 	/* become the new user */
 	if (change_uid (&pwent)) {
 		pam_setcred (pamh, PAM_DELETE_CRED);
 		pam_end (pamh, PAM_ABORT);
 		exit (1);
 	}
-
-	/* now we are done using PAM */
-	pam_end (pamh, PAM_SUCCESS);
-
 #else				/* !USE_PAM */
 	if (!amroot)		/* no limits if su from root */
 		setup_limits (&pwent);
@@ -597,12 +727,6 @@ int main (int argc, char **argv)
 	if (fakelogin) {
 		char *arg0;
 
-#if 0				/* XXX - GNU su doesn't do this.  --marekm */
-		if (!hushed (&pwent)) {
-			motd ();
-			mailcheck ();
-		}
-#endif
 		cp = getdef_str ("SU_NAME");
 		if (!cp)
 			cp = Basename (pwent.pw_shell);
@@ -622,13 +746,21 @@ int main (int argc, char **argv)
 		 */
 
 		argv[-1] = pwent.pw_shell;
+#ifndef USE_PAM
 		(void) execv (pwent.pw_shell, &argv[-1]);
+#else
+		run_shell (pwent.pw_shell, &argv[-1], 0);
+#endif
 		(void) fprintf (stderr, _("No shell\n"));
 		SYSLOG ((LOG_WARN, "Cannot execute %s", pwent.pw_shell));
 		closelog ();
 		exit (1);
 	}
-
+#ifndef USE_PAM
 	shell (pwent.pw_shell, cp);
-	 /*NOTREACHED*/ exit (1);
+#else
+	run_shell (pwent.pw_shell, &cp, 1);
+#endif
+	/* NOT REACHED */
+	exit (1);
 }
